/*
 * HP8562A device support
 */

#include <epicsStdio.h>
#include <devCommonGpib.h>

/******************************************************************************
 *
 * The following define statements are used to declare the names to be used
 * for the dset tables.   
 *
 * A DSET_AI entry must be declared here and referenced in an application
 * database description file even if the device provides no AI records.
 *
 ******************************************************************************/
#define DSET_AI     devAiHP8562A
#define DSET_AO     devAoHP8562A
#define DSET_BO     devBoHP8562A

#include <devGpib.h> /* must be included after DSET defines */

#define TIMEOUT     1.0    /* I/O must complete within this time */
#define TIMEWINDOW  2.0    /* Wait this long after device timeout */

/******************************************************************************
 * Strings used by the init routines to fill in the znam,onam,...
 * fields in BI and BO record types.
 ******************************************************************************/

static char  *mkpkList[] = { "MKPK","MKPK" };
static struct devGpibNames mkpk = { 2,mkpkList,0,1 };

/******************************************************************************
 * Array of structures that define all GPIB messages
 * supported for this type of instrument.
 ******************************************************************************/

static struct gpibCmd gpibCmds[] = {
    /* Param 0 -- Read Marker A amplitude */
    {&DSET_AI, GPIBREAD, IB_Q_HIGH, "MA?", "%lg", 0, 200, NULL, 0, 0, NULL, NULL, NULL},

    /* Param 1 - Marker track off/on */
    {&DSET_BO, GPIBWRITE, IB_Q_HIGH, NULL, "MT%d", 0, 80, NULL, 0, 0, NULL, NULL, NULL},

    /* Param 2 - Input attenuation */
    {&DSET_AO, GPIBWRITE, IB_Q_HIGH, NULL, "AT %g DB", 0, 80, NULL, 0, 0, NULL, NULL, NULL},

    /* Param 3 - Reference Level */
    {&DSET_AO, GPIBWRITE, IB_Q_HIGH, NULL, "RL %g DB", 0, 80, NULL, 0, 0, NULL, NULL, NULL},

    /* Param 4 - Center Frequency */
    {&DSET_AO, GPIBWRITE, IB_Q_HIGH, NULL, "CF %g MHZ", 0, 80, NULL, 0, 0, NULL, NULL, NULL},

    /* Param 5 - Span */
    {&DSET_AO, GPIBWRITE, IB_Q_HIGH, NULL, "SP %g KHZ", 0, 80, NULL, 0, 0, NULL, NULL, NULL},

    /* Param 6 -- Marker frequency */
    {&DSET_AI, GPIBREAD, IB_Q_HIGH, "MKF?", "%lg", 0, 200, NULL, 0, 0, NULL, NULL, NULL},

    /* Param 7 -- Move marker to signal peak */
    {&DSET_BO, GPIBCMD, IB_Q_HIGH, "MKPK", NULL, 0, 200, NULL, 0, 0, NULL, &mkpk, NULL},
};

/******************************************************************************
 * Initialize device support parameters
 *
 *****************************************************************************/
static long init_ai(int parm)
{
    if(parm==0) {
        devSupParms.name = "devHP8562A";
        devSupParms.gpibCmds = gpibCmds;
        devSupParms.numparams = sizeof(gpibCmds)/sizeof(struct gpibCmd);
        devSupParms.timeout = TIMEOUT;
        devSupParms.timeWindow = TIMEWINDOW;
        devSupParms.respond2Writes = -1;
    }
    return(0);
}
